@@ -47,7 +47,8 @@ extern int optind;
 extern int runastrusteduser(), runasrootuser();
 extern void prversion(), doabort();
 extern int mail_priority;
-extern void check_and_print_to();
+extern void check_and_print_to __((FILE *, char*, char*));
+extern int is_xtext_string __((char *));
 
 char *progname;
 
@@ -61,6 +62,7 @@ int D_alloc = 0;	/* memory usage debugging */
 FILE	*mfp = NULL;
 extern char *postoffice; /* At mail_open() */
 
+
 int
 main(argc, argv)
 	int argc;
@@ -78,6 +80,11 @@ main(argc, argv)
 	char	buf[8192];
 	char   *buf2 = NULL;
 	char	*bodytype = NULL;
+	int	usersubmission = 0;
+	char	*submitprotocol = NULL;
+	char	*notificationopt = NULL;
+	char	*returnopt = NULL;
+	char	*envidstr = NULL;
 
 	if ((cp = strrchr(argv[0], '/')) != NULL)
 		progname = ++cp;
@@ -103,7 +110,7 @@ main(argc, argv)
 	configfile = pooption = NULL;
 	if (printq || aliases)
 		goto otherprog;
-	while ((c = getopt(argc, argv,"B:EJb:e:r:f:h:F:C:d:Oo:P:imnq:stvVx")) != EOF) {
+	while ((c = getopt(argc, argv,"B:C:EF:JN:OP:R:UV:b:d:e:r:f:h:imno:p:q:stvx")) != EOF) {
 		switch (c) {
 		case 'B':
 			/* Sendmail 8.7 compability:
@@ -201,6 +208,41 @@ main(argc, argv)
 		case 'O':	/* for NeXT mail */
 			/* default behavior */
 			break;
+		case 'N':
+			notificationopt = optarg;
+			{
+			  char *s = optarg;
+			  while (*s) {
+			    if (strncasecmp(s,"never",5)==0)
+			      s += 5;
+			    else if (strncasecmp(s,"success",7)==0)
+			      s += 7;
+			    else if (strncasecmp(s,"failure",7)==0)
+			      s += 7;
+			    else if (strncasecmp(s,"delay",5)==0)
+			      s += 5;
+			    if (*s == ',') {
+			      ++s;
+			      continue;
+			    }
+			    if (*s != 0) {
+			      fprintf(stderr,"sendmail: illegal -N -option parameter: '%s'\n",optarg);
+			      exit(EX_USAGE);
+			    }
+			  }
+			}
+			break;
+		case 'R':
+			returnopt = optarg;
+			if (strcasecmp(returnopt,"full") != 0 &&
+			    strcasecmp(returnopt,"hdrs") != 0) {
+			  fprintf(stderr,"sendmail: illegal -R -option parameter: '%s'\n",optarg);
+			  exit(EX_USAGE);
+			}
+			break;
+		case 'U':
+			usersubmission = 1;
+			break;
 		case 'v':	/* be verbose */
 			verbose = 1;
 			break;
@@ -208,12 +250,18 @@ main(argc, argv)
 			sprintf(ebp, " -%c", c);
 			ebp += strlen(ebp);
 			break;
+		case 'p':
+			submitprotocol = optarg;
+			break;
 		case 'P':
 			mail_priority = atoi(optarg);
 			break;
 		case 'V':
-			prversion("sendmail");
-			exit(0);
+			envidstr = optarg;
+			if (!is_xtext_string(envidstr)) {
+			  fprintf(stderr,"sendmail: invalid format of -V (envid) parameter: '%s'\n",envidstr);
+			  exit(EX_USAGE);
+			}
 			break;
 		case 'x':	/* Ignore this AIXism */
 			break;
@@ -421,10 +469,17 @@ otherprog:
 		  fprintf(mfp, "fullname %s\n", fullname);
 		if (from != NULL && *from != '\0')
 		  fprintf(mfp, "from %s\n", from);
+		if (envidstr)
+		  fprintf(mfp, "envid %s\n", envidstr);
+		if (returnopt)
+		  fprintf(mfp, "notaryret %s\n", returnopt);
+		if (submitprotocol)
+		  fprintf(mfp, "with %s\n", submitprotocol);
+
 		if (!binary) {
 		  for (; optind < argc; ++optind)
 		    /*fprintf(mfp, "to %s\n", argv[optind]);*/
-		    check_and_print_to(mfp,argv[optind]);
+		    check_and_print_to(mfp,argv[optind],notificationopt);
 		}
 
 		if (fflush(mfp) == EOF
@@ -609,11 +664,12 @@ doabort()
 
 /* haa@cs.hut.fi:  Sometimes address components contain pure junk.. */
 void
-check_and_print_to(mfp, addr)
+check_and_print_to(mfp, addr, notify)
 	FILE *mfp;
-	char *addr;
+	char *addr, *notify;
 {
 	char *copy = NULL, *printme = addr, *from, *to;
+	char *s;
 
 	if (addr == NULL) {
 	  fprintf(stderr, "sendmail: Argument botch: NULL address\n");
@@ -633,6 +689,19 @@ check_and_print_to(mfp, addr)
 	}
 	fprintf(mfp, "to %s\n", printme);
 	if (copy) free(copy);
+	fprintf(mfp, "todsn ORCPT=rfc822;");
+	s = printme;
+	while (*s) {
+	  u_char c = *s;
+	  if ('!' <= c && c <= '~' && c != '+' && c != '=')
+	    putc(c,mfp);
+	  else
+	    fprintf(mfp,"+%02X",c);
+	  ++s;
+	}
+	if (notify)
+	  fprintf(mfp," NOTIFY=%s", notify);
+	putc('\n',mfp);
 }
 
 #if 0
@@ -643,3 +712,27 @@ tmalloc(n)
 	return emalloc(n);
 }
 #endif
+
+int is_xtext_string(str)
+char *str;
+{
+	/* Verify that the input is valid RFC 1981 XTEXT string! */
+
+	while (*str) {
+		unsigned char c = *str;
+		if ('!' <= c && c <= '~' && c != '+' && c != '=')
+		  ; /* is ok! */
+		else if (c == '+') {
+		  c = *++str;
+		  if (!(('0' <= c && c <= '9') || ('A' <= c && c <= 'F')))
+		    return 0; /* Invalid! */
+		  c = *++str;
+		  if (!(('0' <= c && c <= '9') || ('A' <= c && c <= 'F')))
+		    return 0; /* Invalid! */
+		} else {
+		  return 0; /* Is not valid XTEXT string */
+		}
+		++str;
+	}
+	return 1;
+}
