@@ -169,14 +169,16 @@ typedef enum {	normal,		/* nil */
 		eEnvid,		/* SMTP-DSN ENVID string */
 		eFrom,		/* envelope sender address */
 		eFullname,	/* use this as my full name, for local users */
+		eNotaryRet,	/* SMTP-DSN RET token: FULL/HDRS */
 		ePrettyLogin,	/* use this as my pretty login name, ditto. */
 		eRcvdFrom,	/* previous host/user in reverse route */
 		eTo,		/* envelope recipient address */
-		eToDSN,		/* DSN RCPT data entries (NOTIFY,ORCPT,RET) */
+		eToDSN,		/* DSN RCPT data entries (NOTIFY,ORCPT) */
 		eUser,		/* user at previous host in rev route */
 		eVerbose,	/* verbose log filename for "mail -v" */
 		eVia,		/* received via <medium> */
 		eWith,		/* received with <protocol> */
+		eBodytype,	/* The user has given some sendmail -B -option */
 		eEnvEnd		/* Last of the envelope entries "env-end" */
 } HeaderClass;
 
