@@ -19,11 +19,11 @@ struct address {
 	char		**routermxes;	/* [mea] hostpointers from router */
 };
 
-#define _DSN_RET_NO		0x001
-#define _DSN_RET_YES		0x002
+#define _DSN_RET_FULL		0x001
+#define _DSN_RET_HDRS		0x002
 #define _DSN_NOTIFY_SUCCESS	0x004
 #define _DSN_NOTIFY_FAILURE	0x008
-#define _DSN_NOTIFY_ALWAYS	0x010
+#define _DSN_NOTIFY_DELAY	0x010
 #define _DSN_NOTIFY_NEVER	0x020
 
 /* `convertmode' controls the behaviour of the message conversion:
@@ -42,7 +42,7 @@ struct rcpt {
 	struct rcpt	*next;
 	struct address	*addr;		/* addr.link is the sender address */
 	char		*orcpt;		/* DSN ORCPT= string */
-	int		dsnflags;	/* DSN  RET=, and NOTIFY= flags */
+	char		*notify;	/* DSN NOTIFY= flags */
 	char		***newmsgheader; /* message header line pointer ptr
 					   that points to an address of
 					      ctldesc->msgheaders[index]
@@ -63,6 +63,7 @@ struct ctldesc {
 	char		*logident;	/* message id for logging */
 	char		*verbose;	/* file for verbose logging */
 	char		*envid;		/* DSN ENVID data */
+	char		*dsnretmode;	/* DSN RET=-mode */
 	long		msgbodyoffset;	/* offset of message body in msgfile */
 	long		msgsizeestimate; /* Estimate of the msg size */
 	int		msgfd;		/* message file I/O descriptor */
@@ -203,3 +204,8 @@ extern RETSIGTYPE wantout __(());
 
 /* warning.c */
 extern int warning __(());
+
+#ifndef CISTREQ
+#define  CISTREQ(x, y)      (cistrcmp((char *)(x), (char *)(y)) == 0)
+#define  CISTREQN(x, y, n)  (cistrncmp((char *)(x), (char *)(y), n) == 0)
+#endif
