@@ -29,11 +29,7 @@ loginit()
 	fflush(stderr);
 	rewind(stderr);
 	if (freopen(log, "a+", stdout) != stdout
-#ifdef	USE_DUP2
 	    || dup2(FILENO(stdout), FILENO(stderr)) < 0) {	/* sigh */
-#else	/* !USE_DUP2 */
-	    || (close(FILENO(stderr)), dup(FILENO(stdout))) < 0) { /* sigh */
-#endif	/* USE_DUP2 */
 		/* XX: stderr might be closed at this point... */
 		fprintf(stderr, "%s: cannot open log: %s\n", progname, log);
 		return -1;
@@ -45,6 +41,6 @@ loginit()
 #endif	/* F_SETFL */
 	setvbuf(stdout, (char *)NULL, _IOLBF, 0);
 	setvbuf(stderr, (char *)NULL, _IOLBF, 0);
-	SIGNAL_HANDLE(SIGHUP, (SIGNAL_TYPE (*)()) loginit);
+	SIGNAL_HANDLE(SIGHUP, (RETSIGTYPE (*)()) loginit);
 	return 0;
 }
