@@ -15,15 +15,6 @@
 #include <ctype.h>
 #include <pwd.h>
 
-#ifdef	USE_SETUIDX	/* IBM AIXism */
-# include <sys/id.h>
-# ifdef  USE_SETREUID
-#  define setreuid(x,y) setuidx(ID_REAL|ID_EFFECTIVE, y)
-# else
-#  define setuid(y) setuidx(ID_REAL|ID_EFFECTIVE, y)
-# endif
-#endif
-
 #define	DEFTRUSTEDUSER	"daemon"	/* default trusted user if no other */
 
 static int trusteduid = -1;	/* set to trusted uid if we find it */
@@ -83,16 +74,9 @@ runastrusteduser()
 		settrusteduser();
 	if (trusteduid == 0)
 		return 0;		/* trusted uid not found */
-#ifdef	USE_SETREUID
 	setreuid(0, -1);		/* set real uid to root */
 	setreuid(-1, trusteduid);	/* set euid to trusted */
-#else	/* !USE_SETREUID */
-#ifdef	USE_SETEUID
-	seteuid(trusteduid);
-#else	/* !USE_SETEUID */
-	setuid(trusteduid);
-#endif	/* !USE_SETEUID */
-#endif	/* USE_SETREUID */
+
 	return trusteduid;
 }
 
@@ -104,14 +88,7 @@ runasrootuser()
 {
 	if (trusteduid <= 0)
 		return;			/* nothing done before */
-#ifdef	USE_SETREUID
+
 	setreuid(-1, 0);		/* make us effectively root */
 	setreuid(origruid, -1);		/* reset to old real uid */
-#else	/* !USE_SETREUID */
-#ifdef	USE_SETEUID
-	seteuid(0);
-#else	/* !USE_SETEUID */
-	setuid(0);
-#endif	/* !USE_SETEUID */
-#endif	/* USE_SETREUID */
 }
