@@ -3,9 +3,9 @@
  *	This will be free software, but only when it is finished.
  */
 
-#include <stdio.h>
 #include "hostenv.h"
-#ifndef	USE_SETVBUF
+#include <stdio.h>
+#ifndef	HAVE_SETVBUF
 
 #define _IOFBF	0
 #define	_IONBF	04
@@ -28,4 +28,4 @@ setvbuf(fp, buf, type, size)
 		setlinebuf(fp);
 	return (type != _IOLBF && type != _IONBF && type != _IOFBF);
 }
-#endif	/* !USE_SETVBUF */
+#endif	/* !HAVE_SETVBUF */
