@@ -56,7 +56,7 @@ int std_fputs(s, fp) char *s; FILE *fp; { return fputs(s, fp); }
 #if (defined(__GNUC__) && defined(sun) && !defined(__svr4__))
 extern int printf(); /* Sun 4.1.x with old GCC */
 #endif
-int (*std_printf) __((char *fmt, ...)) = (int(*)())printf;
+int (*std_printf) __((char *fmt, ...)) = (int(*)__((char *, ...)))printf;
 #else
 int (*std_printf) __((/*char *fmt, ...*/)) = (int (*) __((/* char *fmt, ... */)))printf;
 #endif
