@@ -25,8 +25,10 @@ extern void ShInitIFS();
 extern void assign();
 extern char **environ;
 extern char *progname;
-extern char *strrchr();
+#ifndef strchr
 extern char *strchr();
+extern char *strrchr();
+#endif
 
 int v_record = 0;	/* record variable accesses */
 int v_changed = 0;	/* a variable in v_accessed was changed */
@@ -349,7 +351,8 @@ v_envinit()
 {
 	struct conscell *s, *e, *tmp;
 	register u_char	**cpp, *cp;
-	int oval, gotpath;
+	int gotpath;
+	memtypes oval;
 
 	oval = stickymem;
 	stickymem = MEM_MALLOC;
@@ -442,7 +445,7 @@ v_export(name)
 	}
 	/* it isn't being exported */
 	if (value == NULL) {
-		int oval = stickymem;
+		memtypes oval = stickymem;
 		stickymem = MEM_MALLOC;
 		value = newstring((u_char *)strsave((char*)name));
 		cdr(value) = conststring((u_char *)"");
