@@ -1266,9 +1266,11 @@ run_listexpand(avl, il)
 	struct siobuf *osiop;
 	FILE *mfp, *fp, f;
 	char buf[4096];
+	char DSNbuf[4096+200];
 	int fd2;
 	char *olderrors = errors_to;
 	volatile FILE *errstr = stderr;
+	char *notary = NULL;
 
 	extern struct conscell *s_value;
 
@@ -1284,17 +1286,32 @@ run_listexpand(avl, il)
 	  switch( il->string[1] ) {
 	    case 'c':
 	      comment = (char*)cdr(il)->string;
+	      if (strchr(comment,'\n') != NULL ||
+		  strchr(comment,'\r') != NULL)
+		errflag = 1;
 	      break;
 	    case 'p':
 	      privilege = atoi((char*)cdr(il)->string);
 	      break;
 	    case 'e':
 	      erroraddress = (char*)cdr(il)->string;
+	      if (strchr(erroraddress,'\n') != NULL ||
+		  strchr(erroraddress,'\r') != NULL)
+		errflag = 1;
 	      break;
 	    case 'E':
 	      if (errors_to != olderrors)
 		free(errors_to);
 	      errors_to = strdup((char*)cdr(il)->string);
+	      if (strchr(errors_to,'\n') != NULL ||
+		  strchr(errors_to,'\r') != NULL)
+		errflag = 1;
+	      break;
+	    case 'N':
+	      notary = (char *)cdr(il)->string;
+	      if (strchr(notary,'\n') != NULL ||
+		  strchr(notary,'\r') != NULL)
+		errflag = 1;
 	      break;
 	    default:
 	      errflag = 1;
@@ -1311,8 +1328,8 @@ run_listexpand(avl, il)
 
 	if (errflag || cnt != 3) {
 		fprintf(stderr,
-			"Usage: %s [ -e error-address ] [ -E errors-to-address ] [-p privilege] [ -c comment ] $attribute $localpart $origaddr < /file/path \n",
-			car(avl)->string);
+			"Usage: %s [ -e error-address ] [ -E errors-to-address ] [-p privilege] [ -c comment ] [ -N notarystring ] $attribute $localpart $origaddr < /file/path \n",
+		car(avl)->string);
 		if (errors_to != olderrors)
 		  free(errors_to);
 		errors_to = olderrors;
@@ -1468,6 +1485,7 @@ run_listexpand(avl, il)
 		char *av[5];
 		int rc;
 		struct conscell *l, *d, *lrc;
+		char *s, *s2, *se;
 
 		buf[0] = 0;
 		pureAddressBuf(buf,sizeof(buf),ap->a_tokens);
@@ -1478,10 +1496,37 @@ run_listexpand(avl, il)
 
 #define use_lapply 1
 #ifdef use_lapply
-		/* XX: We leak these at the moment! */
 		l       = newstring((u_char*)strsave(buf));
 		cdr(l)  = conststring((u_char*)origaddr);
-		cddr(l) = car(attributes);
+		*DSNbuf = 0;
+		if (notary != NULL)
+		  strncpy(DSNbuf,notary,sizeof(DSNbuf)-30);
+		DSNbuf[sizeof(DSNbuf)-30] = 0; /* Be brutal, and chop
+						  off the tail, if it is
+						  too long.. */
+		s2 = strlen(DSNbuf)+DSNbuf;
+		if (s2 > DSNbuf)
+		  strcpy(s2++," ");
+		strcpy(s2,"ORCPT=rfc822;");
+		s2 += strlen(s2);
+		se = DSNbuf + sizeof(DSNbuf)-1; /* BUF-end */
+		s = buf;
+		while (*s) {
+		  u_char c = *s;
+		  if ('!' <= c && c <= '~' && c != '+' && c != '=')
+		    if (s2 < se)
+		      *s2++ = c;
+		  else if (s2 < se) {
+		    sprintf(s2,"+%02X",c);
+		    s2 += 3;
+		  }
+		  ++s;
+		}
+		*s2 = 0;
+		s = build_gensym(privilege,"recipient",DSNbuf,errors_to);
+
+		lrc = newstring((u_char*)strsave(s));
+		cddr(l) = lrc;
 		l = ncons(l);
 
 		deferit = 0;
@@ -1490,6 +1535,7 @@ run_listexpand(avl, il)
 		rc = l_apply("rrouter", l);
 		lrc = s_value;
 		s_value = NULL;
+		s_free_tree(l); /* We can clean up the input list */
 #else
 		lrc = router(ap, privilege, "recipient");
 		rc = (lrc == NULL);
