@@ -6,7 +6,7 @@
 /* LINTLIBRARY */
 
 #include "mailer.h"
-#ifdef	USE_HOSTS
+#ifdef	HAVE_RESOLVER
 #include <netdb.h>
 #include "search.h"
 
@@ -40,7 +40,7 @@ print_hosts(sip, outfp)
 	struct search_info *sip;
 	FILE *outfp;
 {
-#if	!defined(sgi) && !defined(USE_RESOLV)
+#if	!defined(sgi) && !defined(HAVE_RESOLVER)
 	char **cpp;
 	struct hostent *hp;
 
@@ -59,4 +59,4 @@ print_hosts(sip, outfp)
 #endif
 	fflush(outfp);
 }
-#endif	/* USE_HOSTS */
+#endif	/* HAVE_RESOLVER */
