@@ -7,8 +7,8 @@
 
 #include "mailer.h"
 #ifdef	HAVE_NDBM_H
-#ifdef FCNTL_H
-# include FCNTL_H
+#ifdef HAVE_FCNTL_H
+# include <fcntl.h>
 #endif
 #include <ndbm.h>
 #include <sys/file.h>
