@@ -1121,7 +1121,8 @@ sequencer(e, file)
 	int def_uid, header_error, perr;
 	FILE *ofp, *vfp;
 	char vbuf[2048], verbosefile[1024];
-	char *envid = NULL;
+	char *envid;
+	char *notaryret;
 
 	errors_to = NULL; /* to be gotten rid off.. */
 
@@ -1218,6 +1219,12 @@ sequencer(e, file)
 	    && h->h_contents.a->a_pname != NULL) {
 		envid = (char*)h->h_contents.a->a_pname;
 	}
+	FindEnvelope(eNotaryRet);
+	notaryret = NULL;
+	if (h != NULL && h->h_contents.a != NULL
+	    && h->h_contents.a->a_pname != NULL) {
+		notaryret = (char*)h->h_contents.a->a_pname;
+	}
 	FindEnvelope(eFrom);
 	if (h == NULL && e->e_trusted) {
 		/* Perhaps  'channel error' ??? */
@@ -1770,6 +1777,9 @@ sequencer(e, file)
 	if (envid != NULL)
 		fprintf(ofp, "%c%c%s\n",
 			_CF_DSNENVID, _CFTAG_NORMAL, envid);
+	if (notaryret != NULL)
+		fprintf(ofp, "%c%c%s\n",
+			_CF_DSNRETMODE, _CFTAG_NORMAL, notaryret);
 	if (e->e_messageid != NULL)
 		fprintf(ofp, "%c%c%s\n",
 			_CF_LOGIDENT, _CFTAG_NORMAL, e->e_messageid);
