@@ -20,16 +20,16 @@
 #include <fcntl.h>
 #include <sys/file.h>
 #include "mail.h"
-#ifdef	USE_SYSLOG
+#ifdef	HAVE_SYSLOG
 #include <syslog.h>
-#endif	/* USE_SYSLOG */
+#endif	/* HAVE_SYSLOG */
 #include "interpret.h"
 #include "splay.h"
 
 #include "prototypes.h"
 #include "libz.h"
 
-extern int loginit();/* should be SIGNAL_TYPE but we're playing games */
+extern int loginit();/* should be RETSIGTYPE but we're playing games */
 extern int nobody;
 extern int sh_include();
 extern int zshtoplevel();
@@ -52,10 +52,10 @@ static void initialize __((char *configfile, int argc, char *argv[]));
 static void logit __((char *file, char *id, char *from, char *to));
 
 
-char	*progname;
+extern char *progname; /* At zmsh_init() et.friends */
 char	*mailshare;
 char	*myhostname = 0;
-char	*postoffice = NULL;
+extern char *postoffice; /* at mail_open() */
 char	*pidfile = PID_ROUTER;
 char	*log;
 time_t	now;
@@ -257,7 +257,7 @@ main(argc, argv)
 		  }
 		if (nrouters > 1) {
 		  int pgrp, ppid = getpid();
-#ifdef	USE_BSDSETPGRP		/* We assume getpgrp() and setpgrp() calling
+#ifndef	GETPGRP_VOID		/* We assume getpgrp() and setpgrp() calling
 				   conventions match in the machine.. */
 		  pgrp = getpgrp(ppid);	/* BSD  -style */
 #else
