@@ -342,6 +342,49 @@ setenvinfo(e)
 	v_setl("envelopeinfo", plhead);
 }
 
+/*
+ * Build gensym
+ */
+char *
+build_gensym(uid,type,DSNstr,errors_to)
+int uid;
+char *type, *DSNstr, *errors_to;
+{
+	static char gsbuf[30];
+	char buf[20];
+	struct conscell *l, *lc, *tmp;
+
+	/* assemble the default attribute list: (privilege <uid>) */
+	l = conststring((u_char *)"privilege");
+	sprintf(buf, "%d", uid);
+	cdr(l) = newstring((u_char *)strsave(buf));
+	lc = cdr(l);
+	if (type != NULL) {
+		cdr(lc) = conststring((u_char *)"type");
+		lc = cdr(lc);
+		cdr(lc) = newstring((u_char *)strsave(type));
+		lc = cdr(lc);
+	}
+	if (DSNstr != NULL) {
+		cdr(lc) = conststring((u_char *)"DSN");
+		lc = cdr(lc);
+		cdr(lc) = newstring((u_char *)strsave(DSNstr));
+		lc = cdr(lc);
+	}
+	/* See if some "errorsto" definition is available.. */
+	if (errors_to != NULL) {
+		cdr(lc) = conststring((u_char *)"ERR");
+		lc = cdr(lc);
+		cdr(lc) = newstring((u_char *)strsave(errors_to));
+		lc = cdr(lc);
+	}
+	l = ncons(l);
+	sprintf(gsbuf, gs_name, gensym++);
+	/* gX (name in gsbuf) will be freed by free_gensym() later */
+	v_setl(gsbuf, l);
+	return gsbuf;
+}
+
 /*
  * The router function must return three values,
  *	a (channel, host, user) triple.
@@ -361,13 +404,16 @@ router(a, uid, type)
 	struct token *last;
 	struct addr *p;
 	struct conscell *l, *lc, *tmp;
-	char buf[30], gsbuf[30];
+	char buf[30], *gsym;
 	struct notary *DSN = NULL;
+	char *DSNstr;
 
 	if (a == NULL)
 		return NULL;
 	t = last = NULL;
 	DSN = a->a_dsn;
+	DSNstr = NULL;
+	if (DSN) DSNstr = DSN->dsn;
 	for (p = a->a_tokens; p != NULL; p = p->p_next)
 		if (p->p_type == anAddress) {
 			/* link up all address tokens together */
@@ -391,43 +437,16 @@ router(a, uid, type)
 	if (t->t_pname[0] == '<' && TOKENLEN(t) == 1 && t->t_next == NULL)
 		abort();
 
-	/* assemble the default attribute list: (privilege <uid>) */
-	l = conststring((u_char *)"privilege");
-	sprintf(buf, "%d", uid);
-	cdr(l) = newstring((u_char *)buf);
-	lc = cdr(l);
-	if (type != NULL) {
-		cdr(lc) = conststring((u_char *)"type");
-		lc = cdr(lc);
-		cdr(lc) = newstring((u_char *) type );
-		lc = cdr(l);
-	}
-	if (DSN != NULL && DSN->dsn != NULL) {
-		cdr(lc) = conststring((u_char *)"DSN");
-		lc = cdr(lc);
-		cdr(lc) = conststring((u_char *)DSN->dsn);
-		lc = cdr(lc);
-	}
-	/* See if some "errorsto" definition is available.. */
-	if (errors_to != NULL) {
-		cdr(lc) = conststring((u_char *)"ERR");
-		lc = cdr(lc);
-		cdr(lc) = newstring((u_char *)strsave(errors_to));
-		lc = cdr(lc);
-	}
-	l = ncons(l);
-	sprintf(gsbuf, gs_name, gensym++);
-	/* g0 (name in gsbuf) will be freed by free_gensym() later */
-	v_setl(gsbuf, l);
+	gsym = build_gensym(uid,type,DSNstr,errors_to);
 
 	deferit = 0;
 	v_set(DEFER, "");
-	r = s_rewrite(ROUTER, t, (u_char *)gsbuf);
+	r = s_rewrite(ROUTER, t, (u_char *)gsym);
 #if 0
 	if (deferit) {
 		s_free_tree(s_value);
 		s_value = NULL;
-		r = s_rewrite(DEFERENV, t, (u_char *)gsbuf);
+		r = s_rewrite(DEFERENV, t, (u_char *)gsym);
 	}
 #endif
 	if (r != 0 || s_value == NULL || !LIST(s_value)) {
