@@ -40,7 +40,8 @@ extern int errno;
 
 struct not {
 	char *not;
-	char *dsn;
+	char *orcpt;
+	char *notify;
 	char *rcpt;
 	char *rcpntp;
 };
@@ -106,13 +107,13 @@ scnotaryreport(errfp,notary)
 	    typetag = type_local;
 	} else
 	  typetag = type_local;
+	if (notary->orcpt)
+	  fprintf(errfp, "Original-Recipient: %s\n", notary->orcpt);
 	fprintf(errfp, "Final-Recipient: %s; %s\n", typetag, rcpt);
 	fprintf(errfp, "Action: %s\n", action);
 	if (status)
 	  fprintf(errfp, "Status: %s\n", status);
 	fprintf(errfp, "Diagnostic-Code: %s\n", diagnostic);
-	if (notary->dsn)
-	  fprintf(errfp, "X-DSN: %s\n", notary->dsn);
 	if (wtt && wtt[0] != 0)
 	  fprintf(errfp, "Remote-MTA: %s\n", wtt);
 	fprintf(errfp, "\n");
@@ -261,11 +262,92 @@ reporterrs(cfpi)
 	for (i = 0; i < cfp->nlines; ++i, ++lp) {
 	  cp = cfp->contents + *lp;
 
-	  /* Line fmt:
-	     <TAGS> offset ':' headeroffset <TAB> NOTARY-DATA <TAB> MESSAGE <NL> */
-
 	  if (!(*cp == _CF_DIAGNOSTIC && *++cp == _CFTAG_NORMAL))
 	    continue;
+
+	  /* Line fmt:
+	     <TAGS> offset ':' headeroffset ':' drptidx <TAB> NOTARY-DATA <TAB> MESSAGE <NL> */
+
+	  cp = cfp->contents + *lp + 2;
+
+	  byteidx = atoi(cp);
+	  while (*cp  && isascii(*cp) && isdigit(*cp))
+	    ++cp;
+	  headeridx = -1;
+	  if (*cp == ':') {
+	    ++cp;
+	    headeridx = atoi(cp);
+	    while (*cp && isascii(*cp) && (isdigit(*cp) || *cp == '-'))
+	      ++cp;
+	  }
+	  drptidx = -1;
+	  if (*cp == ':') {
+	    ++cp;
+	    drptidx = atoi(cp);
+	    while (isascii(*cp) && (isdigit(*cp) || *cp == '-'))
+	      ++cp;
+	  }
+	  if (*cp == ':') {
+	    int offs = atoi(++cp);
+	    if (offs > 0)
+	      dnsrecipient = cfp->contents + offs;
+	    while (isascii(*cp) && (isdigit(*cp) || *cp == '-'))
+	      ++cp;
+	  }
+	  notary = NULL;
+	  if (*cp == '\t') {
+	    notary = ++cp;
+	    while (*cp && *cp != '\t') ++cp;
+	    notary = strnsave(notary, cp-notary); /* Make a copy of it */
+	    ++cp;
+	  }
+	  rcpntpointer = cfp->contents + byteidx + 2 + _CFTAG_RCPTPIDSIZE;
+	  if (notary != NULL && *notary != 0) {
+	    if (!notaries) {
+	      notaryspc = 8;
+	      notaries = (void*)emalloc(sizeof(struct not)*(notaryspc+1));
+	    }
+	    if (notarycnt >= notaryspc) {
+	      notaryspc += 8;
+	      notaries = (void*)erealloc((void*)notaries,
+					 sizeof(struct not)*(notaryspc+1));
+	    }
+	    notaries[notarycnt].not    = notary;
+	    notaries[notarycnt].orcpt  = NULL;
+	    notaries[notarycnt].notify = NULL;
+	    notaries[notarycnt].rcpt   = dnsrecipient;
+	    notaries[notarycnt].rcpntp = rcpntpointer;
+	    if (drptidx > 0) {
+	      char *d = cfp->contents + drptidx;
+	      while (*d) {
+		while (*d != 0 && (*d == ' ' || *d == '\t')) ++d;
+		if (CISTREQN(d,"ORCPT=",6)) {
+		  notaries[notarycnt].orcpt = d+6;
+		  d += 6;
+		  while (*d != 0 && (*d != ' ' && *d != '\t')) ++d;
+		  if (*d) *d++ = 0;
+		  continue;
+		}
+		if (CISTREQN(d,"NOTIFY=",7)) {
+		  notaries[notarycnt].orcpt = d+7;
+		  d += 7;
+		  while (*d != 0 && (*d != ' ' && *d != '\t')) ++d;
+		  if (*d) *d++ = 0;
+		  continue;
+		}
+		/* XX: Other junk at DSN parameter string ? */
+		while (*d != 0 && (*d != ' ' && *d != '\t')) ++d;
+		if (*d) *d++ = 0;
+		continue;
+	      }
+	      if (notaries[notarycnt].notify != NULL &&
+		  CISTREQN(notaries[notarycnt].notify,"NEVER",5))
+		continue; /* Don't report on this recipient! */
+	    }
+	    ++notarycnt;
+	    notaries[notarycnt].not = NULL;
+	  }
+
 	  if (!wroteheader) {
 	    if (*eaddr == '|' /* Nobody's address starts with a pipe ? */ ||
 		(*eaddr == '/' &&
@@ -323,65 +405,19 @@ reporterrs(cfpi)
 	    }
 	    /* fprintf(errfp, "\n"); -- not needed ?? */
 	    wroteheader = 1;
+
+	  } /* End of failure list processing */
+
+	  if (notaries == NULL) {
+	    /* Oops!  No recipients on which to report anything ?! */
+	    mail_abort(errfp);
+	    close(cfp->fd);
+	    cfp->mid = NULL; /* we don't want to loose the original one! */
+	    free_cfp_memory(cfp);
+	    return;
 	  }
+
 	  
-	  /* Line fmt:
-	     <TAGS> offset ':' headeroffset ':' drptidx <TAB> NOTARY-DATA <TAB> MESSAGE <NL> */
-
-	  cp = cfp->contents + *lp + 2;
-	  byteidx = atoi(cp);
-	  while (*cp  && isascii(*cp) && isdigit(*cp))
-	    ++cp;
-	  headeridx = -1;
-	  if (*cp == ':') {
-	    ++cp;
-	    headeridx = atoi(cp);
-	    while (*cp && isascii(*cp) && (isdigit(*cp) || *cp == '-'))
-	      ++cp;
-	  }
-	  drptidx = -1;
-	  if (*cp == ':') {
-	    ++cp;
-	    drptidx = atoi(cp);
-	    while (isascii(*cp) && (isdigit(*cp) || *cp == '-'))
-	      ++cp;
-	  }
-	  if (*cp == ':') {
-	    int offs = atoi(++cp);
-	    if (offs > 0)
-	      dnsrecipient = cfp->contents + offs;
-	    while (isascii(*cp) && (isdigit(*cp) || *cp == '-'))
-	      ++cp;
-	  }
-	  notary = NULL;
-	  if (*cp == '\t') {
-	    notary = ++cp;
-	    while (*cp && *cp != '\t') ++cp;
-	    notary = strnsave(notary, cp-notary); /* Make a copy of it */
-	    ++cp;
-	  }
-	  rcpntpointer = cfp->contents + byteidx + 2 + _CFTAG_RCPTPIDSIZE;
-	  if (notary != NULL && *notary != 0) {
-	    if (!notaries) {
-	      notaryspc = 8;
-	      notaries = (void*)emalloc(sizeof(struct not)*(notaryspc+1));
-	    }
-	    if (notarycnt >= notaryspc) {
-	      notaryspc += 8;
-	      notaries = (void*)erealloc((void*)notaries,
-					 sizeof(struct not)*(notaryspc+1));
-	    }
-	    notaries[notarycnt].not    = notary;
-	    notaries[notarycnt].dsn    = NULL;
-	    notaries[notarycnt].rcpt   = dnsrecipient;
-	    notaries[notarycnt].rcpntp = rcpntpointer;
-	    if (drptidx > 0) {
-	      notaries[notarycnt].dsn = cfp->contents + drptidx;
-	    }
-	    ++notarycnt;
-	    notaries[notarycnt].not = NULL;
-	  }
-
 	  /* Scan to the start of the message text */
 	  while (isascii(*cp) && isspace(*cp))
 	    ++cp;
@@ -465,9 +501,11 @@ reporterrs(cfpi)
 	      fseek(fp,(off_t)0,0);
 	    }
 	  }
-	  /* Copy out the rest with somewhat more efficient method */
-	  while ((n = fread(buf, sizeof buf[0], sizeof buf, fp)) > 0)
-	    fwrite(buf, sizeof buf[0], n, errfp);
+	  if (!cfp->dsnretmode || CISTREQN(cfp->dsnretmode,"FULL",4)) {
+	    /* Copy out the rest (=body) with somewhat more efficient method */
+	    while ((n = fread(buf, sizeof buf[0], sizeof buf, fp)) > 0)
+	      fwrite(buf, sizeof buf[0], n, errfp);
+	  }
 	  fclose(fp);
 	} else {
 	  fprintf(stderr,"Could not open message body file: '%s'\n",path);
@@ -476,7 +514,7 @@ reporterrs(cfpi)
 	fprintf(errfp, "--%s--\n", boundarystr);
 
 	if (no_error_report > 0)
-	  mail_abort(errfp);
+	  mail_close_alternate(errfp,POSTMANDIR,":error-on-error");
 	else
 	  mail_close(errfp);	/* XX: check for error */
 	close(cfp->fd);
