@@ -100,6 +100,9 @@ extern FILE *statuslog;
 extern void resync_file __((char *filename));
 extern int thread_count_recipients __((void));
 extern time_t mytime __((time_t *));
+extern int dq_insert __((void*, long, char*, int));
+extern int in_dirscanqueue __((void *, long));
+extern char *cfpdirname __((int));
 
 /* threads.c */
 extern struct thread *thread_head, *thread_tail;
