@@ -37,9 +37,7 @@ static int rc_ageorder __((char *key, char *arg, struct config_entry *ce));
 static int rc_deliveryform __((char *key, char *arg, struct config_entry *ce));
 static int rc_overfeed __((char *key, char *arg, struct config_entry *ce));
 
-extern char *strchr();
 extern char *strerror();
-extern char *strrchr();
 extern int errno;
 
 static char *mailbin = NULL;
