@@ -1120,6 +1120,8 @@ slurp(fd, ino)
 		cfp->msgbodyoffset = atoi(s+2);
 	      } else if (*s == _CF_DSNENVID) {
 		/* cfp->envid = strsave(s+2); */
+	      } else if (*s == _CF_DSNRETMODE) {
+		/* cfp->dsnretmode = strsave(s+2); */
 	      } else if (*s == _CF_MESSAGEID) {
 		/* cfp->mid = strsave(s+2); */
 	      } else if (*s == _CF_LOGIDENT) {
@@ -1228,6 +1230,7 @@ static struct ctlfile *vtxprep(cfp, file)
 	struct offsort *offarr;
 	int offspc;
 	long ino;
+	int prevrcpt = -1;
 
 	char fpath[128], path[128], path2[128];
 
@@ -1288,8 +1291,12 @@ static struct ctlfile *vtxprep(cfp, file)
 	  /* Calculate summary info */
 	  if (cp[-1] == _CF_RECIPIENT) {
 	    cfp->rcpnts_total += 1;
-	    if (*cp == _CFTAG_NOTOK)   cfp->rcpnts_failed += 1;
-	    else if (*cp == _CFTAG_OK) cfp->rcpnts_work   += 1;
+	    if (*cp == _CFTAG_NOTOK) {
+	      cfp->rcpnts_failed += 1;
+	      prevrcpt = -1;
+	    } else if (*cp == _CFTAG_OK) {
+	      cfp->rcpnts_work   += 1;
+	    }
 	  }
 	  if (*cp == _CFTAG_NORMAL ||
 	      *cp == '\n' /* This appears for msg-header entries.. */ ) {
@@ -1327,64 +1334,22 @@ static struct ctlfile *vtxprep(cfp, file)
 	      offarr[opcnt].headeroffset = -1;
 	      offarr[opcnt].drptoffset = -1;
 	      offarr[opcnt].sender = latest_sender;
+	      prevrcpt = opcnt;
 	      ++opcnt;
 	      break;
 	    case _CF_RCPTNOTARY:
-	      /* XX: IETF-NOTARY-DRPT DATA! */
-	      offarr[svn].drptoffset = *lp + 2;
-#if 0 /* ---------------------------------------------------------------- */
-	      /* This fragment is pulled from transports/libta/ctlopen.c */
-	      ++s;
-	      while (*s) {
-		while (*s && (*s == ' ' || *s == '\t')) ++s;
-		if (cistrncmp("RET=",s,4)==0) {
-		  s += 4;
-		  if (cistrncmp("YES",s,3)==0) {
-		    rp->dsnflags |= _DSN_RET_YES;
-		    s += 3;
-		  } else if (cistrncmp("NO",s,2)==0) {
-		    rp->dsnflags |= _DSN_RET_NO;
-		    s += 2;
-		  } else 
-		    /* Not YES/NO ??? */
-		    ;
-		  while (*s && *s != ' ' && *s != '\t') ++s;
-		  continue;
-		}
-		if (cistrncmp("NOTIFY=",s,7)==0) {
-		  s += 7;
-		  if (cistrncmp("SUCCESS",s,7)==0) {
-		    rp->dsnflags |= _DSN_NOTIFY_SUCCESS;
-		    s += 7;
-		  } else if (cistrncmp("FAILURE",s,7)==0) {
-		    rp->dsnflags |= _DSN_NOTIFY_FAILURE;
-		    s += 7;
-		  } else if (cistrncmp("ALWAYS",s,6)==0) {
-		    rp->dsnflags |= _DSN_NOTIFY_ALWAYS;
-		    s += 6;
-		  } else if (cistrncmp("NEVER",s,5)==0) {
-		    rp->dsnflags |= _DSN_NOTIFY_NEVER;
-		    s += 5;
-		  } else
-		    /* None of the above ?? */
-		    ;
-		  while (*s && *s != ' ' && *s != '\t') ++s;
-		  continue;
-		}
-		if (cistrncmp("ORCPT=",s,6)==0) {
-		  s += 6;
-		  rp->orcpt = s;
-		  while (*s && *s != ' ' && *s != '\t') ++s;
-		  if (*s) *s++ = 0;
-		  continue;
-		}
-	      }
-#endif /* ---------------------------------------------------------------- */
+	      /* IETF-NOTARY-DRPT DATA! */
+	      if (prevrcpt >= 0)
+		offarr[prevrcpt].drptoffset = *lp + 2;
 	      break;
 	    case _CF_MSGHEADERS:
 	      for (/* we count up.. */; svn < opcnt; ++svn)
 		offarr[svn].headeroffset = *lp + 2;
 	      break;
+	    case _CF_DSNRETMODE:
+	      if (cfp->dsnretmode) free(cfp->dsnretmode);
+	      cfp->dsnretmode = strsave(cp+2);
+	      break;
 	    case _CF_DSNENVID:
 	      if (cfp->envid) free(cfp->envid); /* shouldn't happen.. */
 	      cfp->envid = strsave(cp+2);
@@ -1544,12 +1509,13 @@ static struct ctlfile *vtxprep(cfp, file)
 	pvpp = &head;
 	for (i = 0; i < opcnt; ++i) {
 	  channel = bcp + offarr[i].offset /* - 2 */;
+	  while (*channel == ' ') ++channel; /* Skip possible space(s) */
 	  if ((echannel = strchr(channel, ' ')) == NULL) /* error! */
 	    continue;
 	  *echannel = '\0';
 	  strlower(channel);
 	  host = echannel + 1;
-	  while (*host == ' ') ++host;
+	  while (*host == ' ') ++host; /* Skip space */
 #if 1
 	  /* [mea]   channel ((mx.target)(mx.target mx.target)) rest.. */
 	  cp = host;
