@@ -891,11 +891,6 @@ int ok, justfree;
 	  return; /* WE ARE IDLE! */
 	}
 
-	/* if (proc->cmdbuf[0] != 0) {
-	   if (verbose) printf(" ... NONE, feeding incomplete..\n");
-	   return;
-	   } */
-
 	if (!justfree && proc->fed == 0 && proc->vertex != NULL) {
 	  if (verbose) printf(" ... NONE, current one has not been fed..\n");
 	  return; /* Current one has not been (completely) fed..	*/
@@ -930,6 +925,12 @@ int ok, justfree;
 	  return;
 	}
 
+	/* We won't change threads, until the buffer has been flushed */
+	/* .. except if ordered by 'justfree' -- damn resync.. */
+	if (!justfree && proc->cmdlen != 0) {
+	  if (verbose) printf(" ... NONE, this thread empty, and feeding incomplete..\n");
+	  return;
+	}
 
 	/* Pick another thread in our ring */
 	thr = thr->nextthg;
@@ -1086,14 +1087,10 @@ time_t retrytime;
 	    /* ... and now expire it! */
 	    /* this MAY invalidate also the THREAD object! */
 	    if (thr->jobs > 1) {
-if (nvtx == NULL)
-fprintf(stderr,"threads:reschedule() - expiring with thr->jobs=%d, nvtx == NULL!\n",
-	thr->jobs);
+	      /* if (nvtx == NULL) fprintf(stderr,"threads:reschedule() - expiring with thr->jobs=%d, nvtx == NULL!\n",thr->jobs); */
 	      expire(vtx,index);
 	    } else {
-if (nvtx != NULL)
-fprintf(stderr,"threads:reschedule() - expiring with thr->jobs=%d, nvtx != NULL!\n",
-	thr->jobs);
+	      /* if (nvtx != NULL) fprintf(stderr,"threads:reschedule() - expiring with thr->jobs=%d, nvtx != NULL!\n",	thr->jobs); */
 	      expire(vtx,index);
 	      thr = NULL; /* The THR-pointed object is now invalid */
 	    }
@@ -1307,7 +1304,8 @@ if (verbose) printf("idle_cleanup()\n");
 
 	    while (p != NULL) {
 	      ++idlecnt;
-	      if (thg->cep->idlemax + p->hungertime < now) {
+	      if (thg->cep->idlemax + p->hungertime < now &&
+		  p->cmdlen == 0) {
 		/* It is old enough -- ancient, one might say.. */
 
 		/* Close the command channel, let it die itself.
@@ -1401,7 +1399,7 @@ FILE *fp;
 	  thr_once = 1;
 	  for (thr = thg->thread; thr && (thr_once || thr != thg->thread); thr = thr->nextthg) {
 	    int width;
-	    width = fprintf(fp,"\t%s/%s/%d",
+	    width = fprintf(fp,"    %s/%s/%d",
 			    /* thr->vertices->orig[L_CHANNEL]->name, */ thr->channel,
 			    /* thr->vertices->orig[L_HOST]->name, */ thr->host,
 			    thr->thgrp->withhost);
@@ -1444,19 +1442,19 @@ FILE *fp;
 		      (thr->vertices->ce_pending == L_CHANNEL ? " channelwait" : " threadwait"));
 	    fprintf(fp,"\n");
 	  }
-	  fprintf(fp,"\t\tThreads: %3d",thg->threads);
+	  fprintf(fp,"\tThreads: %4d",thg->threads);
 	  if (thg->threads != cnt)
 	    fprintf(fp,"/%d",cnt);
-	  fprintf(fp," Rcpts: %4d",jobsum);
-	  fprintf(fp," Procs: %2d",thg->transporters);
+	  fprintf(fp," Rcpts: %5d",jobsum);
+	  fprintf(fp," Procs: %3d",thg->transporters);
 	  /* 	  if (thg->transporters != procs)
 		  fprintf(fp,"/%d",procs);		*/
 	  cnt = 0;
 	  for (p = thg->idleproc; p != 0; p = p->next) ++cnt;
-	  fprintf(fp," Idle: %2d",thg->idlecnt);
+	  fprintf(fp," Idle: %3d",thg->idlecnt);
 	  if (thg->idlecnt != cnt)
 	    fprintf(fp,"/%d",cnt);
-	  fprintf(fp," Plim: %2d Flim: %3d ",thg->ce.maxkidThreads,thg->ce.overfeed);
+	  fprintf(fp," Plim: %3d Flim: %3d ",thg->ce.maxkidThreads,thg->ce.overfeed);
 	  jobtotal  += jobsum;
 	  threadsum += thg->threads;
 	  fprintf(fp,"\n");
