@@ -13,7 +13,9 @@
 #include <sys/param.h>
 #include "scheduler.h"
 #include "mail.h"
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
+#endif
 #include <sys/stat.h>
 #include <errno.h>
 #include <sys/file.h>
@@ -71,8 +73,10 @@ idle_child(proc)
 struct procinfo *proc;
 {
 	int len, rc;
-	/* we are NOT to be called while there is something left in the cmdbuf[] ! */
+	/* we are NOT to be called while there is something
+	   left in the cmdbuf[] ! */
 	if (proc->cmdlen != 0) {
+	  proc->cmdbuf[proc->cmdlen] = 0;
 	  fprintf(stderr,"idle_child(proc->cmdbuf=\"%s\") -> abort()!\n",proc->cmdbuf);
 	  fflush(stderr);
 	  /* abort(); */ /* Hmm.. ??? */
@@ -87,9 +91,11 @@ struct procinfo *proc;
 	    /* Some real failure :-( */
 	    pipes_shutdown_child(proc->tofd);
 	    proc->tofd = -1;
-	    proc->vertex->proc = NULL;
-	    proc->thread->proc = NULL;
-	    proc->thread       = NULL;
+	    if (proc->vertex)
+	      proc->vertex->proc = NULL;
+	    if (proc->thread)
+	      proc->thread->proc = NULL;
+	    proc->thread         = NULL;
 	    proc->cmdlen = 0;
 	    return;
 	  }
@@ -117,6 +123,11 @@ struct procinfo *proc;
 	if (proc->cmdlen != 0 && proc->tofd >= 0) {
 	  /* We have some leftovers from previous feed..
 	     .. feed them now.  */
+
+	  /* fprintf(stderr,
+	     "flushing to child pid %d, cmdlen=%d, cmdbuf='%s'\n",
+	     proc->pid, proc->cmdlen, proc->cmdbuf);  */
+
 	  rc = write(proc->tofd, proc->cmdbuf, proc->cmdlen);
 	  if (rc != proc->cmdlen) {
 	    if (rc < 0 && (errno != EAGAIN && errno != EINTR &&
@@ -169,10 +180,25 @@ struct procinfo *proc;
 
 	vtx = proc->vertex;
 
-	if (proc->thg->withhost)
-	  sprintf(cmdbuf, "%s\n", vtx->cfp->mid);
-	else
-	  sprintf(cmdbuf, "%s\t%s\n", vtx->cfp->mid, proc->ho->name);
+	if (!vtx) { /* No active vertex left, child is in inactive
+		       chain, but has leftover stuff at the output
+		       buffer */
+	  flush_child(proc);
+	  return;
+	}
+
+	if (vtx->cfp->dirind > 0) {
+	  char *d = cfpdirname(vtx->cfp->dirind);
+	  if (proc->thg->withhost)
+	    sprintf(cmdbuf, "%s/%s\n", d, vtx->cfp->mid);
+	  else
+	    sprintf(cmdbuf, "%s/%s\t%s\n", d, vtx->cfp->mid, proc->ho->name);
+	} else {
+	  if (proc->thg->withhost)
+	    sprintf(cmdbuf, "%s\n", vtx->cfp->mid);
+	  else
+	    sprintf(cmdbuf, "%s\t%s\n", vtx->cfp->mid, proc->ho->name);
+	}
 
 	len = strlen(cmdbuf);
 
@@ -587,7 +613,6 @@ time_t timeout;
 	  fprintf(stderr,"**** recursed into mux()! ***\n");
 	  return 0;
 	}
-	in_select = 1;
 
 	mytime(&now);
 
@@ -614,9 +639,13 @@ time_t timeout;
 	}
 	if (maxf < 0)
 	  return -1;
+
 	++maxf;
 	/*fprintf(stderr, "about to select on %x [%d]\n",
 			  mask.fds_bits[0], maxf); */
+
+	in_select = 1;
+
 	if ((n = select(maxf, &rdmask, &wrmask, NULL, &tv)) < 0) {
 	  int err = errno;
 	  /* fprintf(stderr, "got an interrupt (%d)\n", errno); */
@@ -691,9 +720,9 @@ time_t timeout;
 		queryipccheck();
 	      }
 	  }
+	  in_select = 0;
 	}
 	/* fprintf(stderr, "return from mux\n"); */
-	in_select = 0;
 	return 0;
 }
 
