@@ -635,7 +635,7 @@ expire(vp, index)
 	  /* addres / action / status / diagnostic / wtt */
 	  sprintf(buf,"%s\001%s\001%s\001%s",
 		  "\003", /* XX: recipient address! XX: MAGIC INFO! */
-		  "failure",
+		  "failed",
 		  "4.0.0 (unspecified timeout failure)",
 		  "smtp; 400 (Expired after ");
 	  saytime((u_long)(vp->ce_expiry - vp->cfp->ctime), buf, 0);
