@@ -500,11 +500,11 @@ main(argc, argv)
 	    openlog("smtpserver", 0);
 #endif /* LOG_MAIL */
 
-	    if (ident_flag != 0) {
-	      setrfc1413ident(0);
-	    } else {
+	    if (ident_flag != 0)
+	      setrfc1413ident(0,&raddr);
+	    else
 	      strcpy(ident_username, "IDENT-NOT-QUERIED");
-	    }
+
 	    sprintf(ident_username+strlen(ident_username),
 		    "@%s (port %d)",rhostname, rport);
 #ifdef	LOG_INFO
@@ -674,11 +674,11 @@ main(argc, argv)
 		openlog("smtpserver", 0);
 #endif /* !LOG_MAIL */
 
-		if (ident_flag != 0) {
-		  setrfc1413ident(0);
-		} else {
+		if (ident_flag != 0)
+		  setrfc1413ident(msgfd,&raddr);
+		else
 		  strcpy(ident_username, "IDENT-NOT-QUERIED");
-		}
+
 		sprintf(ident_username+strlen(ident_username),
 			"@%s (port %d)", rhostname, rport);
 #ifdef	LOG_INFO
@@ -898,6 +898,87 @@ void s_setup(fd)
 }
 
 
+
+
+
+char *xtext_string(str)
+char *str;
+{
+	/* Verify that the input is valid RFC 1981 XTEXT string! */
+	char *str0 = str;
+
+	while (*str) {
+	  unsigned char c = *str;
+
+	  if (c == ' ' || c == '\t')
+	    break;
+
+	  if ('!' <= c && c <= '~' && c != '+' && c != '=')
+	    ; /* is ok! */
+	  else if (c == '+') {
+	    c = *++str;
+	    if (!(('0' <= c && c <= '9') || ('A' <= c && c <= 'F'))) {
+	      goto err;
+	    }
+	    c = *++str;
+	    if (!(('0' <= c && c <= '9') || ('A' <= c && c <= 'F'))) {
+	      goto err;
+	    }
+	  } else {
+	    goto err;
+	  }
+	  ++str;
+	}
+	if (str == str0) {
+	err:
+	  rfc821_error_ptr = str;
+	  return NULL;
+	}
+	return str;
+}
+
+char *rfc822atom(str)
+char *str;
+{
+	char *str0 = str;
+	while (*str != 0) {
+	  u_char c = *str;
+	  /* XX: SHOULD do this with a charmap test! */
+	  if (c <= ' ' || c > 126 ||
+	      c == '(' || c == ')' ||
+	      c == '<' || c == '>' ||
+	      c == ',' || c == ';' ||
+	      c == ':' || c == '\\' ||
+	      c == '"' || c == '[' || c == ']') {
+	    if (str > str0)
+	      return str;
+	    goto err;
+	  }
+	  ++str;
+	}
+	if (str == str0) {
+	err:
+	  rfc821_error_ptr = str;
+	  return NULL;
+	}
+	return str;
+}
+
+char *orcpt_string(str)
+char *str;
+{
+	/* Verify that the input is valid RFC 1981 XTEXT string! */
+	str = rfc822atom(str);
+
+	if (str == NULL) return NULL;
+	if (*str != ';') {
+	  rfc821_error_ptr = str-1;
+	  return NULL;
+	}
+	return xtext_string(++str);
+}
+
+
 /* SMTP-server verb subroutines */
 
 void
@@ -992,10 +1073,10 @@ char *buf, *cp;
 	  type(-250,NULL, "8BITMIME");
 	  type(-250,NULL, "PIPELINING");
 	  type(-250,NULL, "CHUNKING"); /* RFC 1830 */
-	  type(-250,NULL, "ENHANGEDSTATUSCODES");
+	  type(-250,NULL, "ENHANCEDSTATUSCODES");
 	  type(-250,NULL, "EXPN");
 	  type(-250,NULL, "VRFY");
-	  type(-250,NULL, "XDSN"); /* XX: Change to "DSN" when all parts of it function! */
+	  type(-250,NULL, "X-DSN"); /* XX: Change to "DSN" when all parts of it function! */
 	  type(-250,NULL, "X-RCPTLIMIT 10000"); /* VERY HIGH figure, normal is 100 */
 	  type(-250,NULL, "X-TURNME");
 	  type(-250,NULL, "ETRN");
@@ -1013,6 +1094,7 @@ int insecure;
 	char *s;
 	int rc;
 	char *drpt_envid;
+	char *drpt_ret;
 	char *bodytype = NULL;
 
 	/* For ESMTP SIZE-option use we need to know
@@ -1087,13 +1169,31 @@ int insecure;
 	++cp;		/* Skip the initial '<' */
 	*(s-1) = 0;	/* Scrub the final  '>' */
 
-	/* BODY=8BITMIME SIZE=nnnn ENVID=xxxxxx */
+	/* BODY=8BITMIME SIZE=nnnn ENVID=xxxxxx RET=xxxx */
 	sizeoptval = -1;
 	sizeoptsum = -1;
 	drpt_envid = NULL;
+	drpt_ret   = NULL;
 	rc = 0;
 	while (*s) {
 	  while (*s && (*s == ' '||*s == '\t')) ++s;
+	  if (CISTREQN("RET=",s,4)) {
+	    if (drpt_ret) {
+	      type(501,m554,"RET-param double defined!");
+	      return;
+	    }
+	    s += 4;
+	    drpt_ret = s;
+	    if (CISTREQN("FULL",s,4) ||
+		CISTREQN("HDRS",s,4))
+	      s += 4;
+	    if (*s && *s != ' ' && *s != '\t') {
+	      type(501,m454,"RET-param data error");
+	      return;
+	    }
+	    if (*s) *s++ = '\0';
+	    continue;
+	  }
 	  if (CISTREQN("BODY=",s,5)) {
 	    /* Actually we do not use this data... */
 	    s += 5;
@@ -1113,7 +1213,7 @@ int insecure;
 	      s += 4;
 	    }
 	    if (*s && *s != ' ' && *s != '\t') {
-	      type(455, m454, "BODY-param data error, must be one of: 8BITMIME/BINARYMIME/7BIT");
+	      type(455, m554, "BODY-param data error, must be one of: 8BITMIME/BINARYMIME/7BIT");
 	      rc = 1;
 	      break;
 	    }
@@ -1134,7 +1234,7 @@ int insecure;
 	      ++s;
 	    }
 	    if (*s && *s != ' ' && *s != '\t') {
-	      type(501,m454,"SIZE-param data error");
+	      type(501,m554,"SIZE-param data error");
 	      rc = 1;
 	      break;
 	    }
@@ -1148,23 +1248,23 @@ int insecure;
 	      break;
 	    }
 	    drpt_envid = s+6;
-	    while (*s && *s != ' ' && *s != '\t') {
-	      if (*s < 33 || *s > 126) {
-		type(501,m554,"ENVID data contains illegal characters!");
-		rc = 1;
-		break;
-	      }
-	      ++s;
+	    s = xtext_string(s+6);
+	    if (s == NULL) {
+	      type821err(-501,m554,buf,"Invalid ENVID value '%s'", drpt_envid);
+	      type(501,m554,"ENVID data contains illegal characters!");
+	      rc = 1;
+	      break;
 	    }
 	    *s++ = 0;
-	    if (strlen(drpt_envid) < 6) {
-	      type(501,m454,"ENVID= missing data!");
+	    if (*drpt_envid == 0) {
+	      type(501,m554,"ENVID= without data!");
 	      rc = 1;
 	      break;
 	    }
 	    continue;
 	  }
 	  type(555,m454,"Unknown MAIL FROM:<> parameter: %s",s);
+	  rc = 1;
 	  break;
 	}
 	if (rc != 0) return; /* Error(s) in previous loop.. */
@@ -1210,6 +1310,8 @@ int insecure;
 	  fprintf(mfp, "from <%s>\n", cp);
 	if (drpt_envid != NULL)
 	  fprintf(mfp, "envid %s\n", drpt_envid);
+	if (drpt_ret != NULL)
+	  fprintf(mfp, "notaryret %s\n", drpt_ret);
 
 	sender_ok = 0; /* Set it, when we are sure.. */
 
@@ -1233,12 +1335,13 @@ int insecure;
 	rcpt_count = 0;
 }
 
+
 void
 smtp_rcpt(buf,cp)
 char *buf, *cp;
 {
 	char *s;
-	char *drpt_notify, *drpt_ret, *drpt_orcpt;
+	char *drpt_notify, *drpt_orcpt;
 
 	if (state != Recipient && state != RecipientOrData) {
 	  switch (state) {
@@ -1299,7 +1402,6 @@ char *buf, *cp;
 	}
 
 	drpt_notify = NULL;
-	drpt_ret    = NULL;
 	drpt_orcpt  = NULL;
 
 	while (*s) {
@@ -1312,14 +1414,19 @@ char *buf, *cp;
 	    }
 	    drpt_notify = s;
 	    s += 7;
-	    if (CISTREQN("SUCCESS",s,7))
-	      s += 7;
-	    else if (CISTREQN("FAILURE",s,7))
-	      s += 7;
-	    else if (CISTREQN("ALWAYS",s,6))
-	      s += 6;
-	    else if (CISTREQN("NEVER",s,5))
-	      s += 5;
+	    while (*s) {
+	      if (CISTREQN("SUCCESS",s,7))
+		s += 7;
+	      else if (CISTREQN("FAILURE",s,7))
+		s += 7;
+	      else if (CISTREQN("DELAY",s,5))
+		s += 5;
+	      else if (CISTREQN("NEVER",s,5))
+		s += 5;
+	      if (*s != ',')
+		break;
+	      ++s;
+	    }
 	    if (*s && *s != ' ' && *s != '\t') {
 	      type(455,m454,"NOTIFY-param data error");
 	      return;
@@ -1327,37 +1434,20 @@ char *buf, *cp;
 	    if (*s) *s++ = '\0';
 	    continue;
 	  }
-	  if (CISTREQN("RET=",s,4)) {
-	    if (drpt_ret) {
-	      type(501,m554,"RET-param double defined!");
-	      return;
-	    }
-	    drpt_ret = s;
-	    s += 4;
-	    if (CISTREQN("YES",s,3))
-	      s += 3;
-	    else if (CISTREQN("NO",s,2))
-	      s += 2;
-	    if (*s && *s != ' ' && *s != '\t') {
-	      type(501,m454,"RET-param data error");
-	      return;
-	    }
-	    if (*s) *s++ = '\0';
-	    continue;
-	  }
 	  if (CISTREQN("ORCPT=",s,6)) {
 	    if (drpt_orcpt) {
 	      type(501,m554,"ORCPT-param double defined!");
 	      return;
 	    }
 	    drpt_orcpt = s;
-	    while (*s && *s != ' ' && *s != '\t')
-	      ++s;
-	    if (*s) *s++ = '\0';
-	    if (strlen(drpt_orcpt) < 7) {
+	    s = orcpt_string(s+6);
+	    if (s == NULL) {
+	      type821err(-501,m454,buf,"Invalid ORCPT value '%s'", drpt_orcpt);
 	      type(501,m454,"ORCPT-param data error!");
 	      return;
 	    }
+	    if (s && *s != 0) *s = '\0';
+	    ++s;
 	    continue;
 	  }
 	  type(555,"Unknown RCPT TO:<> parameter: %s",s);
@@ -1379,13 +1469,24 @@ char *buf, *cp;
 	}
 
 	/* IETF-NOTARY DSN data: */
-	if (drpt_orcpt || drpt_notify || drpt_ret) {
-	  fputs("todsn",mfp);
-	  if (drpt_notify) fprintf(mfp, " %s", drpt_notify);
-	  if (drpt_ret)    fprintf(mfp, " %s", drpt_ret);
-	  if (drpt_orcpt)  fprintf(mfp, " %s", drpt_orcpt);
-	  fputs("\n",mfp);
+	fputs("todsn",mfp);
+	if (drpt_notify) fprintf(mfp, " %s", drpt_notify);
+	if (drpt_orcpt) {
+	  fprintf(mfp, " %s", drpt_orcpt);
+	} else {
+	  char *s = cp;
+	  fputs(" ORCPT=rfc822;",mfp);
+	  while (*s) {
+	    u_char c = *s;
+	    if ('!' <= c && c <= '~' && c != '+' && c != '=')
+	      putc(c,mfp);
+	    else
+	      fprintf(mfp,"+%02X",c);
+	    ++s;
+	  }
 	}
+	putc('\n',mfp);
+
 	/* Normal "RCPT TO:<>" data: */
 	fprintf(mfp, "to <%s>\n", cp);
 	if (sizeoptval < 0)
@@ -1426,7 +1527,7 @@ char *name, *cp;
 	fprintf(mfp,"%c%c%s\n",_CF_TURNME,_CFTAG_NORMAL,cp);
 	/* printf("050-My uid=%d/%d\r\n",getuid(),geteuid()); */
 	runasrootuser();
-	if (mail_close_scheduler(mfp)) {
+	if (mail_close_alternate(mfp,TRANSPORTDIR,"")) {
 	  type(452,m400,"Failed to initiate TURNME request;  Permission denied?");
 	} else {
 	  type(-250,m200,"A TURNME request is initiated - lets hope the system");
@@ -2679,8 +2780,8 @@ type821err(code,status,inbuf,msg,s1,s2,s3)
 	printf("%03d-%s ", code < 0 ? -code : code,status);
 	if (logfp != NULL)
 	  fprintf(logfp, "%dw\t%03d-%s ", pid, code,status);
-	if (strlen(inbuf)>3)
-	  while (*s && s < rfc821_error_ptr) {
+	if (((int)(rfc821_error_ptr - s)) < 200)
+	  while (s < rfc821_error_ptr) {
 	    ++s;
 	    putc(' ',stdout);
 	    if (logfp != NULL)
@@ -3025,26 +3126,29 @@ mvbdata(out,msg, incount, statep)
 }
 
 static void
-setrfc1413ident(sd)
-unsigned int sd;
+setrfc1413ident(sd,raddr)
+int sd;
+struct sockaddr_in *raddr;
 {
     char *cp;
     struct in_addr	ident_inlocal;
     struct in_addr	ident_inremote;
     unsigned short	ident_local;
     unsigned short	ident_remote;
+    int rc;
 
-    strncpy(ident_username, "UNKNOWN", MAXHOSTNAMELEN);
-    if (ident_fd2(sd, &ident_inlocal, &ident_inremote,
-		 &ident_local, &ident_remote) == -1) {
-	    return;
+    if ((rc = ident_fd2(sd, &ident_inlocal, &ident_inremote,
+			&ident_local, &ident_remote)) != 0) {
+      sprintf(ident_username, "IDENT-FD2-FAULT-%d-%d", rc, errno);
+      return;
     } else {
-	    cp = ident_tcpuser3(&ident_inlocal, &ident_inremote,
-			       ident_local,   ident_remote,
-			       IDENT_TIMEOUT);
-	    if ( cp != (char *) NULL ) {
-		    strncpy(ident_username, cp, MAXHOSTNAMELEN);
-	    }
+      cp = ident_tcpuser3(&ident_inlocal, &ident_inremote,
+			  ident_local,   ident_remote,
+			  IDENT_TIMEOUT);
+      if (cp != NULL)
+	strncpy(ident_username, cp, MAXHOSTNAMELEN);
+      else
+	strncpy(ident_username, "IDENT-CALL-FAULT", MAXHOSTNAMELEN);
     }
 }
 
