@@ -325,7 +325,7 @@ sendmessage(msgf, myname)
 
 	fflush(tmpf);
 	fclose(f);
-	fseek(tmpf,0,0); /* rewind */
+	fseek(tmpf,(off_t)0,0); /* rewind */
 	if (fileno(tmpf) != 0)
 	  dup2(fileno(tmpf),0); /* Make sure it is stdin! */
 
