@@ -212,7 +212,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	char *mfpath;
 	int i, n;
 	struct address *ap;
-	struct rcpt *rp = NULL;
+	struct rcpt *rp = NULL, *prevrp = NULL;
 	struct stat stbuf;
 	char ***msgheaders = NULL;
 	char ***msgheaderscvt = NULL;
@@ -330,9 +330,10 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	d.rcpnts_total = 0;
 	d.rcpnts_remaining = 0;
 	d.rcpnts_failed = 0;
-	d.logident = "none";
-	d.envid   = NULL;
-	d.verbose = NULL;
+	d.logident   = "none";
+	d.envid      = NULL;
+	d.dsnretmode = NULL;
+	d.verbose    = NULL;
 
 	/* run through the file and set up the information we need */
 	for (i = 0; i < n; ++i) {
@@ -355,8 +356,12 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	    ++s;
 	    /* Calculate statistics .. Scheduler asks for it.. */
 	    d.rcpnts_total += 1;
-	    if (*s == _CFTAG_NOTOK)   d.rcpnts_failed    += 1;
-	    else if (*s != _CFTAG_OK) d.rcpnts_remaining += 1;
+	    if (*s == _CFTAG_NOTOK) {
+	      d.rcpnts_failed    += 1;
+	      prevrp = NULL;
+	    } else if (*s != _CFTAG_OK) {
+	      d.rcpnts_remaining += 1;
+	    }
 
 	    if (*s != _CFTAG_NORMAL || d.senders == NULL)
 	      break;
@@ -409,56 +414,34 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	    rp->next = d.recipients;
 	    rp->desc = &d;
 	    rp->orcpt = NULL;
-	    rp->dsnflags = 0;
+	    rp->notify = NULL;
 	    d.recipients = rp;
 	    rp->status = EX_OK;
 	    rp->newmsgheader = NULL;
+	    prevrp = rp;
 	    break;
 	  case _CF_RCPTNOTARY:
 	    /*  IETF-NOTARY-DSN  DATA */
 	    ++s;
-	    while (*s) {
-	      while (*s && (*s == ' ' || *s == '\t')) ++s;
-	      if (cistrncmp("RET=",s,4)==0) {
-		s += 4;
-		if (cistrncmp("YES",s,3)==0) {
-		  rp->dsnflags |= _DSN_RET_YES;
-		  s += 3;
-		} else if (cistrncmp("NO",s,2)==0) {
-		  rp->dsnflags |= _DSN_RET_NO;
-		  s += 2;
-		} else 
-		  /* Not YES/NO ??? */
-		  ;
-		while (*s && *s != ' ' && *s != '\t') ++s;
-		continue;
-	      }
-	      if (cistrncmp("NOTIFY=",s,7)==0) {
-		s += 7;
-		if (cistrncmp("SUCCESS",s,7)==0) {
-		  rp->dsnflags |= _DSN_NOTIFY_SUCCESS;
+	    if (prevrp != NULL) {
+	      while (*s) {
+		while (*s && (*s == ' ' || *s == '\t')) ++s;
+		if (cistrncmp("NOTIFY=",s,7)==0) {
 		  s += 7;
-		} else if (cistrncmp("FAILURE",s,7)==0) {
-		  rp->dsnflags |= _DSN_NOTIFY_FAILURE;
-		  s += 7;
-		} else if (cistrncmp("ALWAYS",s,6)==0) {
-		  rp->dsnflags |= _DSN_NOTIFY_ALWAYS;
+		  prevrp->notify = s;
+		  while (*s && *s != ' ' && *s != '\t') ++s;
+		  if (*s) *s++ = 0;
+		  continue;
+		}
+		if (cistrncmp("ORCPT=",s,6)==0) {
 		  s += 6;
-		} else if (cistrncmp("NEVER",s,5)==0) {
-		  rp->dsnflags |= _DSN_NOTIFY_NEVER;
-		  s += 5;
-		} else
-		  /* None of the above ?? */
-		  ;
+		  prevrp->orcpt = s;
+		  while (*s && *s != ' ' && *s != '\t') ++s;
+		  if (*s) *s++ = 0;
+		  continue;
+		}
+		/* XX: BOO! Unknown value! */
 		while (*s && *s != ' ' && *s != '\t') ++s;
-		continue;
-	      }
-	      if (cistrncmp("ORCPT=",s,6)==0) {
-		s += 6;
-		rp->orcpt = s;
-		while (*s && *s != ' ' && *s != '\t') ++s;
-		if (*s) *s++ = 0;
-		continue;
 	      }
 	    }
 	    break;
@@ -534,6 +517,9 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	  case _CF_DSNENVID:
 	    d.envid = s+2;
 	    break;
+	  case _CF_DSNRETMODE:
+	    d.dsnretmode = s+2;
+	    break;
 	  case _CF_BODYOFFSET:
 	    d.msgbodyoffset = (long)atoi(s+2);
 	    break;
