@@ -23,14 +23,15 @@
 #endif	/* L_SET */
 
 int
-lockaddr(fd, map, offset, was, new)
+lockaddr(fd, map, offset, was, new, file, host)
 	int	fd;
 	char	*map;
 	long	offset;
 	int	was;
 	int	new;
+	const char *file, *host;
 {
-	char	lockbuf[/* 1+_CFTAG_RCPTPIDSIZE+1 */ 16]; /* Alpha alignments.. */
+	char	lockbuf[16];
 	int	newlock = 0;
 
 #ifdef USE_FCNTLLOCK
@@ -85,8 +86,12 @@ lockaddr(fd, map, offset, was, new)
 #ifdef USE_FCNTLLOCK
 	      /* Using FCNTL region locking */
 #else
+	      static int mypid = -1;
+	      if (mypid < 0)
+		mypid = getpid();
+
 	      /* Mark the lock with client process-id */
-	      sprintf(lockbuf+1,"%*d",_CFTAG_RCPTPIDSIZE,getpid());
+	      sprintf(lockbuf+1,"%*d",_CFTAG_RCPTPIDSIZE,mypid);
 	      if (map)
 		memcpy(map+offset, lockbuf, _CFTAG_RCPTPIDSIZE+1);
 	      else if (write(fd,lockbuf,
@@ -117,6 +122,6 @@ lockaddr(fd, map, offset, was, new)
 	  }
 	  return 1;
 	}
-	warning("lockaddr: expected '%c' found '%c'\n", was, new);
+	warning("lockaddr: file '%s' host '%s' expected '%c' found '%c'\n", file, host, was, new);
 	return 0;
 }
