@@ -640,7 +640,7 @@ uid_t uid;
 	  if ((pw = getpwuid(uid)) == NULL) {
 	    notaryreport(file,"failed",
 			 "5.2.1 (Target file has no known owner)",
-			 "510 (Target file has no known owner)");
+			 "x-local; 510 (Target file has no known owner)");
 	    DIAGNOSTIC(rp, EX_NOPERM,
 		       "File \"%s\" has no owner", file);
 	    return 1;
@@ -657,7 +657,7 @@ uid_t uid;
 	case L_MAXTRYS:
 	  notaryreport(file,"delayed",
 		       "4.4.5 (Mailbox unlocking fails)",
-		       "450 (Mailbox unlocking fails)");
+		       "x-local; 450 (Mailbox unlocking fails)");
 	  DIAGNOSTIC(rp, EX_TEMPFAIL, "\"%s\" is locked", file);
 	  return 1;
 	case L_NAMELEN:
@@ -694,7 +694,7 @@ uid_t uid;
 	  sprintf(mbuf, "\"%s\": %s", file, strerror(errno));
 	  notaryreport(file,"delayed",
 		       "4.4.5 (File locking with dotlock failed)",
-		       "450 (File locking with dotlock failed)");
+		       "x-local; 450 (File locking with dotlock failed)");
 	  DIAGNOSTIC(rp, errno == EBUSY ? EX_TEMPFAIL :
 		     (errno == EACCES ? EX_NOPERM : EX_UNAVAILABLE),
 		     "can't dotlock %s", mbuf);
@@ -714,7 +714,7 @@ char *file;
 	  alarm(0);
 	  notaryreport(file,"delayed",
 		       "4.4.5 (File locking with NFS lock failed)",
-		       "450 (File locking with NFS lock failed)");
+		       "x-local; 450 (File locking with NFS lock failed)");
 	  DIAGNOSTIC(rp, EX_TEMPFAIL,
 		     "nfs lock() can't nfslock \"%s\"", file);
 	  return 1;
@@ -737,7 +737,7 @@ char *file;
 	  alarm(0);
 	  notaryreport(file,"delayed",
 		       "4.4.5 (File locking with  lockf  failed)",
-		       "450 (File locking with  lockf  failed)");
+		       "x-local; 450 (File locking with  lockf  failed)");
 	  DIAGNOSTIC(rp, EX_TEMPFAIL,
 		     "can't lockf() \"%s\"", file);
 	  return 1;
@@ -758,7 +758,7 @@ char *file;
 	  alarm(0);
 	  notaryreport(file,"delayed",
 		       "4.4.5 (File locking with  flock  failed)",
-		       "450 (File locking with  flock  failed)");
+		       "x-local; 450 (File locking with  flock  failed)");
 	  DIAGNOSTIC(rp, EX_TEMPFAIL,
 		     "can't flock() \"%s\"", file);
 	  return 1;
@@ -814,7 +814,7 @@ deliver(dp, rp, timestring)
 	  if (uid == nobody) {
 	    notaryreport("?program?", "failed",
 			 "5.2.1 (Mail to program disallowed w/o proper privileges)",
-			 "550 (Mail to program disallowed w/o proper privileges)");
+			 "x-local; 550 (Mail to program disallowed w/o proper privileges)");
 	    DIAGNOSTIC(rp, EX_UNAVAILABLE,
 		       "mail to program disallowed", 0);
 	    return;
@@ -829,13 +829,13 @@ deliver(dp, rp, timestring)
 	    if (propably_x400(rp->addr->user)) {
 	      notaryreport(rp->addr->user,"failed",
 			   "5.1.4 (this feels like a misplaced X.400 address -- no support for them)",
-			   "550 (this feels like a misplaced X.400 address -- no support for them)");
+			   "x-local; 550 (this feels like a misplaced X.400 address -- no support for them)");
 	      DIAGNOSTIC(rp, EX_UNAVAILABLE,
 			 "This appears to be a misplaced X.400 address, no support for them", 0);
 	    } else {
 	      notaryreport(rp->addr->user,"failed",
 			   "5.2.1 (Mail to file disallowed w/o proper privileges)",
-			   "550 (mail to file disallowed w/o proper privileges)");
+			   "x-local; 550 (mail to file disallowed w/o proper privileges)");
 	      DIAGNOSTIC(rp, EX_UNAVAILABLE,
 			 "mail to file disallowed", 0);
 	    }
@@ -882,13 +882,13 @@ deliver(dp, rp, timestring)
 	      if (propably_x400(rp->addr->user)) {
 		notaryreport(rp->addr->user,"failed",
 			     "5.1.4 (this feels like a misplaced X.400 address -- no support for them)",
-			     "550 (this feels like a misplaced X.400 address -- no support for them)");
+			     "x-local; 550 (this feels like a misplaced X.400 address -- no support for them)");
 		DIAGNOSTIC(rp, EX_UNAVAILABLE,
 			   "This appears to be a misplaced X.400 address, no support for them", 0);
 	      } else {
 		notaryreport(rp->addr->user,"failed",
 			     "5.1.1 (User does not exist)",
-			     "550 (User does not exist)");
+			     "x-local; 550 (User does not exist)");
 		if (at) *at = 0;
 		if (plus) *plus = '+';
 		DIAGNOSTIC(rp, EX_NOUSER,
@@ -921,7 +921,7 @@ deliver(dp, rp, timestring)
 	    char *mailbox = getzenv("MAILBOX");
 	    notaryreport(rp->addr->user,"failed",
 			 "4.3.5 (System mailbox configuration is wrong, we are in deep trouble..)",
-			 "466 (System mailbox configuration is wrong!  No such directory!  Aargh!)");
+			 "x-local; 466 (System mailbox configuration is wrong!  No such directory!  Aargh!)");
 	    DIAGNOSTIC(rp, EX_TEMPFAIL,
 		       "System mailbox configuration is wrong!  No such directory (%s)!  Aargh!",maildirs[0]);
 #ifdef  LOG_PID
@@ -971,7 +971,7 @@ deliver(dp, rp, timestring)
 	if (exstat(rp, file, &st, lstat) < 0) {
 	  notaryreport(rp->addr->user,"failed",
 		       "4.2.0 (User's mailbox disappeared, will retry)",
-		       "466 (User's mailbox disappeared, will retry)");
+		       "x-local; 466 (User's mailbox disappeared, will retry)");
 	  DIAGNOSTIC(rp, EX_TEMPFAIL,
 		     "mailbox file \"%s\" disappeared", file);
 	  return;
@@ -984,7 +984,7 @@ deliver(dp, rp, timestring)
 	if (strcmp(file,"/dev/null") == 0) {
 	  notaryreport(rp->addr->user,"delivery",
 		       "2.2.0 (delivery successfull)",
-		       "250 (Delivered successfully)");
+		       "x-local; 250 (Delivered successfully)");
 	  DIAGNOSTIC(rp, EX_OK, (char *)NULL, 0);
 	  return;
 	}
@@ -993,7 +993,7 @@ deliver(dp, rp, timestring)
 	  /* XX: may want to deliver to named pipes */
 	  notaryreport(rp->addr->user,"failed",
 		       "5.2.1 (Attempting to deliver into non-regular file)",
-		       "500 (Attempting to deliver into non-regular file)");
+		       "x-local; 500 (Attempting to deliver into non-regular file)");
 	  DIAGNOSTIC(rp, EX_UNAVAILABLE,
 		     "attempted delivery to special file \"%s\"",
 		     file);
@@ -1003,7 +1003,7 @@ deliver(dp, rp, timestring)
 	if (st.st_nlink > 1) {
 	  notaryreport(rp->addr->user,"failed",
 		       "4.2.1 (Destination file ambiguous)",
-		       "500 (Destination file has more than one name..)");
+		       "x-local; 500 (Destination file has more than one name..)");
 	  DIAGNOSTIC(rp, EX_UNAVAILABLE,
 		     "too many links to file \"%s\"", file);
 	  return;
@@ -1038,7 +1038,7 @@ deliver(dp, rp, timestring)
 	    rp->status = EX_UNAVAILABLE;
 	  notaryreport(file,"failed",
 		       "4.2.1 (File open for append failed)",
-		       "500 (File open for append failed)");
+		       "x-local; 500 (File open for append failed)");
 	  DIAGNOSTIC(rp, rp->status, fmtbuf, file);
 	  setrootuid(rp);
 	  return;
@@ -1061,7 +1061,7 @@ deliver(dp, rp, timestring)
 	    s2.st_nlink != 1) {
 	    notaryreport(file,"delayed",
 			 "4.4.5 (Lost race for mailbox file)",
-			 "450 (Lost race for mailbox file)");
+			 "x-local; 450 (Lost race for mailbox file)");
 	    DIAGNOSTIC(rp, EX_TEMPFAIL,
 		       "lost race for mailbox \"%s\"", file);
 	    close(fdmail);
@@ -1239,7 +1239,7 @@ deliver(dp, rp, timestring)
 	  fclose(fp);		/* this closes fdmail */
 	  notaryreport(rp->addr->user,"delivery",
 		       "2.2.0 (Delivered successfully)",
-		       "250 (Delivered successfully)");
+		       "x-local; 250 (Delivered successfully)");
 	  DIAGNOSTIC(rp, EX_OK, (char *)NULL, 0);
 	}
 #if	defined(HAVE_SOCKET)
@@ -1318,7 +1318,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    || writeheaders(rp,fp,"\n",convert_qp,0) < 0) {
 	  notaryreport(rp->addr->link->user, "failed",
 		       "4.2.2 (Write to user's mailbox failed)",
-		       "500 (Write to user's mailbox failed)");
+		       "x-local; 500 (Write to user's mailbox failed)");
 	  DIAGNOSTIC(rp, EX_IOERR,
 		     "header write to \"%s\" failed", file);
 #ifdef HAVE_FTRUNCATE
@@ -1495,6 +1495,17 @@ program(dp, rp, timestring)
 	cp += strlen(cp) + 1;
 	sprintf(cp, "MAILSHARE=%s", s);
 	env[i++] = cp;
+	if (rp->orcpt) {
+	  cp += strlen(cp) + 1;
+	  sprintf(cp, "ORCPT=%s", rp->orcpt);
+	  env[i++] = cp;
+	}
+	if (dp->envid) {
+	  cp += strlen(cp) + 1;
+	  sprintf(cp, "ENVID=%s", dp->envid);
+	  env[i++] = cp;
+	}
+
 	env[i] = NULL;
 
 	/* now we can fork off and run the command... */
@@ -1658,7 +1669,7 @@ creatembox(rp, filep, uid, gid, pw)
 	if (fd == -1) {
 	  notaryreport(rp->addr->user, "failed",
 		       "4.3.1 (can't create user spool mailbox file)",
-		       "466 (can't create user spool mailbox file)");
+		       "x-local; 466 (can't create user spool mailbox file)");
 	  DIAGNOSTIC(rp, EX_CANTCREAT, "can't create \"%s\"", *filep);
 	}
 	/* otherwise the message was printed by createfile() */
@@ -1684,7 +1695,7 @@ createfile(rp, file, uid)
 	    *cp = '\0';
 	    if (exstat(rp, file, &st, stat) < 0) {
 	      *cp = '/';
-	      sprintf(msg,"466 (*INVOCATION BUG* Can't create user spool mailbox file: \"%s\", Directory stat() error: %s)",
+	      sprintf(msg,"x-local; 466 (*INVOCATION BUG* Can't create user spool mailbox file: \"%s\", Directory stat() error: %s)",
 		      file,strerror(errno));
 	      *cp = 0;
 	      notaryreport(rp->addr->user, "failed",
@@ -1697,7 +1708,7 @@ createfile(rp, file, uid)
 	      if (!setupuidgid(rp, uid, st.st_gid)) {
 		notaryreport(rp->addr->user, "failed",
 			     "4.3.5 (The mailbox configuration is faulty)",
-			     "466 (*INVOCATION BUG* The mailbox directory is group writable, but can't change my gid to it)");
+			     "x-local; 466 (*INVOCATION BUG* The mailbox directory is group writable, but can't change my gid to it)");
 		DIAGNOSTIC(rp, i,
 			   "failed changing group id to create file in \"%s\"",
 			   file);
@@ -1728,7 +1739,7 @@ createfile(rp, file, uid)
 	else /* if (saverrno == ENOENT) */
 	  i = EX_CANTCREAT;
 	/* convoluted to maintain 4 arguments to DIAGNOSTIC */
-	sprintf(msg, "466 (error [%s] creating \"%%s\")", strerror(saverrno));
+	sprintf(msg, "x-local; 466 (error [%s] creating \"%%s\")", strerror(saverrno));
 	notaryreport(rp->addr->user, "failed",
 		     "4.3.0 (Other mailsystem error)",
 		     msg);
