@@ -399,7 +399,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  for (rp = startrp; rp != endrp; rp = rp->next) {
 	    notaryreport(rp->addr->user,"failed",
 			 "4.3.0 (Out of system resources, pipe creation failed)",
-			 "400 (pipe creation error, out of system resources ?)");
+			 "x-local; 400 (pipe creation error, out of system resources ?)");
 	    diagnostic(rp, EX_OSERR, 0,
 		       "cannot create pipe from \"%s\"",
 		       mp->command);
@@ -410,7 +410,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  for (rp = startrp; rp != endrp; rp = rp->next) {
 	    notaryreport(rp->addr->user,"failed",
 			 "4.3.0 (Out of system resources, pipe creation failed)",
-			 "400 (pipe creation error, out of system resources ?)");
+			 "x-local; 400 (pipe creation error, out of system resources ?)");
 	    diagnostic(rp, EX_OSERR, 0,
 		       "cannot create pipe to \"%s\"",
 		       mp->command);
@@ -448,7 +448,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  for (rp = startrp; rp != endrp; rp = rp->next) {
 	    notaryreport(rp->addr->user,"failed",
 			 "4.3.0 (Out of system resources, fork failed)",
-			 "400 (fork failure, out of system resources ?)");
+			 "x-local; 400 (fork failure, out of system resources ?)");
 	    diagnostic(rp, EX_OSERR, 0, "cannot fork");
 	  }
 	  return;
@@ -556,7 +556,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	    notaryreport(rp->addr->user,"failed",
 			 /* Could indicate: 4.3.1 - mail system full ?? */
 			 "4.3.0 (Write to target failed for some reason)",
-			 "400 (Write to target failed for some reason)");
+			 "x-local; 400 (Write to target failed for some reason)");
 	    diagnostic(rp, i, 0, "write error");
 	  }
 	  /* just to make sure nothing will get delivered */
@@ -612,11 +612,11 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  if (i == EX_OK)
 	    notaryreport(rp->addr->user, "relayed",
 			 "2.2.0",
-			 "250 Delivered");
+			 "x-local; 250 Delivered");
 	  else
 	    notaryreport(rp->addr->user, "failed",
 			 "4.3.0",
-			 "430 Unknown error");
+			 "x-local; 430 Unknown error");
 	  diagnostic(rp, i, 0, "%s", buf);
 	}
 	/* XX: still need to deal with MO_STRIPQUOTES */
