@@ -210,6 +210,7 @@ static const char FAILED[] = "failed";
 #define ESMTP_8BITMIME   0x0002
 #define ESMTP_DSN        0x0004
 #define ESMTP_PIPELINING 0x0008
+#define ESMTP_ENHSTATUS  0x0010
 
 int  ehlo_capabilities = 0;
 long ehlo_sizeval      = 0;
@@ -839,10 +840,13 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	  s += strlen(s);
 	}
 	/* DSN parameters ... */
-	if ((ehlo_capabilities & ESMTP_DSN) &&
-	    startrp->desc->envid != NULL) {
-	  sprintf(s," ENVID=%s",startrp->desc->envid);
-	  s += strlen(s);
+	if (ehlo_capabilities & ESMTP_DSN) {
+	  if (startrp->desc->envid != NULL) {
+	    sprintf(s," ENVID=%s",startrp->desc->envid);
+	    s += strlen(s);
+	  }
+	  if (startrp->desc->dsnretmode != NULL)
+	    sprintf(s, "RET=%s", startrp->desc->dsnretmode);
 	}
 
 	smtp_flush(); /* Flush in every case */
@@ -902,21 +906,11 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	  s = SMTPbuf + strlen(SMTPbuf);
 
 	  if (ehlo_capabilities & ESMTP_DSN) {
-	    if (rp->dsnflags & _DSN_RET_NO)
-	      strcat(s, " RET=NO");
-	    else if (rp->dsnflags & _DSN_RET_YES)
-	      strcat(s, " RET=YES");
-
-	    if (rp->dsnflags & _DSN_NOTIFY_SUCCESS)
-	      strcat(s, " NOTIFY=SUCCESS");
-	    else if (rp->dsnflags & _DSN_NOTIFY_FAILURE)
-	      strcat(s, " NOTIFY=FAILURE");
-	    else if (rp->dsnflags & _DSN_NOTIFY_ALWAYS)
-	      strcat(s, " NOTIFY=ALWAYS");
-	    else if (rp->dsnflags & _DSN_NOTIFY_NEVER)
-	      strcat(s, " NOTIFY=NEVER");
-
-	    s += strlen(s);
+	    if (rp->notify) {
+	      strcat(s, " NOTIFY=");
+	      strcat(s,rp->notify);
+	      s += strlen(s);
+	    }
 	    if (rp->orcpt != NULL) {
 	      sprintf(s, " ORCPT=%s", rp->orcpt);
 	    }
@@ -1538,9 +1532,11 @@ char *buf;
 	if (r != NULL) *r = 0;
 	if (strcmp(buf,"8BITMIME")==0) {
 	  ehlo_capabilities |= ESMTP_8BITMIME;
-	} else if (strcmp(buf,"XDSN")==0 ||
+	} else if (strcmp(buf,"X-DSN")==0 ||
 		   strcmp(buf,"DSN")==0) {
 	  ehlo_capabilities |= ESMTP_DSN;
+	} else if (strncmp(buf,"ENHANCEDSTATUSCODES",19)==0) {
+	  ehlo_capabilities |= ESMTP_ENHSTATUS;
 	} else if (strncmp(buf,"PIPELINING",10)==0) {
 	  ehlo_capabilities |= ESMTP_PIPELINING;
 	} else if (strncmp(buf,"SIZE ",5)==0 ||
