@@ -11,10 +11,10 @@
 #include <ctype.h>
 #include <sys/types.h>
 #include <fcntl.h>
-#ifdef HAVE_NDBM
+#ifdef HAVE_NDBM_H
 #include <ndbm.h>
 #else
-#ifdef HAVE_GDBM
+#ifdef HAVE_GDBM_H
 #include <gdbm.h>
 #else
 	error:error:Only NDBM and GDBM supported! (NDBM primarily)
@@ -39,10 +39,10 @@ int errno;
 {
   fprintf(stderr,"Usage: %s database.name [infilename|-]\n",prog);
   fprintf(stderr,"  If no infilename is defined, database.name is assumed.\n");
-#ifdef HAVE_NDBM
+#ifdef HAVE_NDBM_H
   fprintf(stderr,"  (NDBM appends  .pag, and .dir  into actual db file names..)\n");
 #else /* GDBM.. */
-  fprintf(stderr,"  (GDBM DOES NOT append .pag, nor .dir  into actual db file names..)\n");
+  fprintf(stderr,"  (GDBM appends .pag, to the actual db file name..)\n");
 #endif
   fprintf(stderr," Error now: %s",err);
   if (errno > 0 && errno < sys_nerr)
@@ -58,7 +58,7 @@ char *argv[];
 {
   char *dbasename = NULL;
   FILE *infile = NULL;
-#ifdef HAVE_NDBM
+#ifdef HAVE_NDBM_H
   DBM *dbmfile;
 #else
   GDBM_FILE dbmfile;
@@ -78,10 +78,12 @@ char *argv[];
 
   if (infile == NULL) usage(argv[0],"bad infile",errno);
 
-#ifdef HAVE_NDBM
+#ifdef HAVE_NDBM_H
   dbmfile = dbm_open(dbasename, O_RDWR|O_CREAT|O_TRUNC, 0644);
 #else
   /* Play loose .. don't do syncs while writing */
+  dbasename = strcpy(malloc(strlen(dbasename)+8),dbasename);
+  strcat(dbasename,".pag"); /* ALWAYS append this */
   dbmfile = gdbm_open(dbasename, 0, GDBM_NEWDB|GDBM_FAST, 0644);
 #endif
   if (dbmfile == NULL)
@@ -98,7 +100,7 @@ char *argv[];
 void
 create_dbase(infile,dbmfile)
 FILE *infile;
-#ifdef HAVE_NDBM
+#ifdef HAVE_NDBM_H
 DBM *dbmfile;
 #else
 GDBM_FILE dbmfile;
@@ -140,7 +142,7 @@ GDBM_FILE dbmfile;
 	  dat.dptr  = s;
 	  dat.dsize = strlen(s)+1;
 
-#ifdef HAVE_NDBM
+#ifdef HAVE_NDBM_H
 	  rc = dbm_store(dbmfile, key, dat, DBM_INSERT);
 #else
 	  rc = gdbm_store(dbmfile, key, dat, GDBM_INSERT);
